//
//  Test SW for testing the Nokia N95-8G Display with an Arduino Due Board
//
//  written from Christian K. 02/2014
//
//  based on the work of
//  http://andybrown.me.uk/wk/2012/11/03/nokia-n95-8gb-lcd/
//  and
//  http://mikrocontrollerprojekte.blogspot.de/
//
#include <Parallel.h>
#include <smc.h>


#include <N958G.h>
#include <fonts/allFonts.h>

// display wirering, how to connect the display for this scetch
// LED backlight power is pin 13
// other 3.3V pins are for the diplay

#define WR  45
#define RD  8  
#define CS  7  
#define RS  4  
#define RES 5  
#define TE  44 
#define KBB 29  
#define LCDPW 13  // power for Display (Not used)

#define D0 34
#define D1 35
#define D2 36
#define D3 37
#define D4 38
#define D5 39
#define D6 40
#define D7 41


N958G Display(CS, RS, WR, RES, LCDPW, WHITE);


void setup() {

  Serial.begin(9600);
  Serial.println("Hello");

  // put your setup code here, to run once:
  pinMode(LCDPW, OUTPUT);
  digitalWrite(LCDPW, HIGH);  

  // Configure parallel bus for NCS1, A0, and NWE (no read signal)
  Parallel.begin(PARALLEL_BUS_WIDTH_8, PARALLEL_CS_1, 6, 0, 1);

  // Configure conservative bus timings, fill screen needs 302ms
  //Parallel.setAddressSetupTiming(5,1,5,1);
  //Parallel.setPulseTiming(50,60,50,60);
  //Parallel.setCycleTiming(110,110);

  // Configure moderate bus timings, fill screen needs 166ms
  Parallel.setAddressSetupTiming(5,1,5,1);
  Parallel.setPulseTiming(10,20,10,20);
  Parallel.setCycleTiming(55,55);

  // Configure fast bus timings, fill screen needs 165ms
  // this fast timing does not bring a real speed advantage
  //Parallel.setAddressSetupTiming(3,1,3,1);
  //Parallel.setPulseTiming(5,8,5,8);
  //Parallel.setCycleTiming(15,15);
  
  pinMode(TE, OUTPUT);
  digitalWrite(RD, HIGH);  
  pinMode(RD, OUTPUT); 
  digitalWrite(RD, HIGH);
  pinMode(CS, OUTPUT);  
  digitalWrite(CS, HIGH);
  pinMode(RES, OUTPUT);  
  digitalWrite(RES, LOW);  
  pinMode(KBB, OUTPUT);
  digitalWrite(KBB, LOW);  

  Display.init();
}

void loop() {
  uint32_t duration;

  duration=micros();  

  // put your main code here, to run repeatedly:
  Display.fillScreen(WHITE);

  Display.fillRect(210,300,10,10,BLUE);
  Display.fillRect(180,270,10,10,ORANGE);
  Display.fillRect(1950,240,10,10,RED);
  Display.fillCircle(90,240,60,YELLOW);

  Display.selectFont(Arial_14);

  Display.putS(10,10,"Nokia N95 8GB LCD mit Arduino Due",BLACK);
  Display.putS(10,30,"Anzeigeflaeche: 57 x 42 mm - 2.8''",BLUE);
  Display.putS(10,50,"Aufloesung: 320 x 240 Pixel - QVGA",CYAN);
  
  Display.selectFont(CalBlk36);
  Display.drawRoundRect(20,118,200,98,6,GREEN);
  Display.drawRoundRect(19,117,202,100,6,GREEN);
  int16_t tmp;
  tmp=(Display.width - Display.stringWidth("Danke"))/2;
  Display.putS(tmp, 120, "Danke", RED);
  tmp=(Display.width - Display.stringWidth("Fabian"))/2;
  Display.putS(tmp, 180, "Fabian", BLACK);

  Display.drawLine(1,60,40,100,RED);
  Display.drawFastLine(80,70,123,BLUE,0);
  
  digitalWrite(CS, LOW);
  uint16_t len=10;
  Display.setAddrWindow(30,30,30+len,30+1);
  while (len--)  {
      Display.writeData((GREEN & 0x00FF0000) >> 16);	// R
      Display.writeData((GREEN & 0x0000FF00) >> 8);	// G
      Display.writeData(GREEN & 0x000000FF);		// B   
  }
  digitalWrite(CS, HIGH);

  duration=micros()-duration;
  Serial.print("duration: "); Serial.println(duration);

  delay(500);
}
