// Graphics library by ladyada/adafruit 
// MIT license

#ifndef N958G_H
#define N958G_H

#include <Arduino.h>
#include "Parallel.h"

// Colors

#define WHITE		0xffffff
#define BLACK		0x000000
#define GREEN		0x00ff00
#define YELLOW		0xffff00
#define CYAN		0x00ffff
#define RED		0xff0000
#define BLUE		0x0000ff
#define ORANGE 		0xFFA500


typedef const uint8_t* Font_t;  	

#define FontRead *

class N958G {
 public:
  N958G(const uint8_t CS, const uint8_t RS, const uint8_t WR, const uint8_t RST, const uint8_t PWR, const uint32_t color);

  void init(void);

  void setBacklight(const uint8_t percent);

  // drawing primitives!
  void setBackgroundColor(const uint32_t color);
  uint32_t getBackgroundColor(void) { return (_backgroundColor); };

  void fillScreen(const uint32_t color);
  void drawPixel(const uint16_t x, const uint16_t y, const uint32_t color);

  void drawLine(int16_t x, int16_t y, int16_t x1, int16_t y1, uint32_t color);
  void drawVerticalLine(const uint16_t x0, const uint16_t y0, uint16_t length, const uint32_t color);
  void drawHorizontalLine(const uint16_t x0, const uint16_t y0,uint16_t length,const  uint32_t color);
  void drawFastLine(const uint16_t x0, const uint16_t y0, uint16_t l,const uint32_t color, const uint8_t flag);

  void drawRect(const uint16_t x, const uint16_t y, const uint16_t w, const uint16_t h, const uint32_t color);
  void fillRect(uint16_t x, uint16_t y, const uint16_t w, const uint16_t h, const uint32_t color);
  void drawRoundRect(const uint16_t x, const uint16_t y, const uint16_t width, const uint16_t height, 
                           const uint16_t radius, const uint32_t color);

  void drawCircle(const uint16_t x0, const uint16_t y0, const uint16_t r, const uint32_t color);
  void fillCircle(uint16_t x0, uint16_t y0, uint16_t r, uint32_t color);

// support for arduino GLCD fonts
  void selectFont(const Font_t font);
  uint8_t getFontHeight(void);
  uint16_t putChar(uint16_t x, const uint16_t y, uint8_t c, const uint32_t color);
  uint16_t putS(uint16_t x, const uint16_t y, const char *str, const uint32_t color);
  uint16_t putS(uint16_t x, const uint16_t y, const String &str, const uint32_t color); // for Arduino String Class

  uint8_t charWidth(uint8_t c);
  uint16_t stringWidth(const char* str);
  uint16_t stringWidth(const String &str);  // for arduino String class
// end support for arduino GLCD fonts

  static const uint16_t width = 240;
  static const uint16_t height = 320;

  void setAddrWindow(const uint16_t x0, const uint16_t y0, const uint16_t x1, const uint16_t y1);

  inline void writeCommand(const uint8_t command){  Parallel.write(0x00, command);  };
  inline void writeData(const uint8_t data) {  Parallel.write(0x20, data); };

 private:
  uint8_t _cs, _rs, _wr, _res, _pwr;
  uint32_t _backgroundColor;
  Font_t  Font;    // Arduino GLCD fontS
};

#endif

// end of file
